


# python3 3d_animation.py <vib_num>
# ver 1.0






from matplotlib import pyplot as plt
import numpy as np
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import animation
import re

from matplotlib.patches import FancyArrowPatch
from mpl_toolkits.mplot3d import proj3d


import sys



from scipy.io import mmread


prefix = 'AlPO4'
filename = 'OK_AlPO4_B3LYP_TZVP_NONE'




A = mmread(prefix+filename+'_forceconst.mtx')

Fone = A.todense()
rows = np.shape(Fone)[0]
cols = np.shape(Fone)[1]
print( 'rows,cols =',rows,cols)




which = int(sys.argv[1])


speedup = 5.0



atomhash = {"1":"H","6":"C","7":"N","8":"O","15":"P","13":"Al"}
#atomhash = {"H":"1","C":"6","N":"7","O":"8"}

colorhash = {"1":"gray","6":"black","7":"blue","8":"red","15":"orange","13":"gray"}




with open(prefix+filename+'_atoms.txt') as soub:
    atoms = soub.readline().strip().split(',')

colors = [colorhash[l] for l in atoms ]






#fig = plt.figure()

fig = plt.figure(figsize=(9, 7), dpi=1920/16)

ax = fig.add_subplot(projection='3d')




f = open(prefix+filename+"_vibrations.txt", "r")
numatoms = int(f.readline()) 
print(numatoms)


pos = np.zeros((3,numatoms))

for i in range(numatoms):
  pomo = re.split(r'\s+',f.readline())
  pos[:,i] = pomo[3:6].copy()


print(pos)



for now in range(which):
   
   frek = str(f.readline()).split(" ")
   print(frek[2])
   amp = np.zeros((3,numatoms))

   for i in range(numatoms):
      pomo = re.split(r'\s+',f.readline())
      amp[:,i] = pomo[2:5].copy()



print(amp)



def gen(n,atom):
    phi = 0
    while phi < 2*np.pi:
        yield np.array([pos[0,atom]+amp[0,atom]*np.sin(speedup*phi), pos[1,atom]+amp[1,atom]*np.sin(speedup*phi), pos[2,atom]+amp[2,atom]*np.sin(speedup*phi)])
        phi += 2*np.pi/n



def update(num, data, lines):

    plt.cla()

    for line, walk in zip(lines, data):
      line.set_data(walk[num-3:num,:2].T)
      line.set_3d_properties(walk[num-3:num,2])


    for i in range(numatoms):
      for j in range(numatoms):
        if j>i:

          pomo = Fone[3*i:3*(i+1),3*j:3*(j+1)]
          soucet = np.sum(abs(pomo))
#          print(soucet)

          xline = np.linspace(data[i][num][0],data[j][num][0], 100)
          yline = np.linspace(data[i][num][1],data[j][num][1], 100)
          zline = np.linspace(data[i][num][2],data[j][num][2], 100)
          if soucet>0.1:
            ax.plot3D(xline, yline, zline, c=cmap(soucet))

    aktualx = [data[l][num][0] for l in range(numatoms) ]
    aktualy = [data[l][num][1] for l in range(numatoms) ]
    aktualz = [data[l][num][2] for l in range(numatoms) ]

    ax.scatter3D(aktualx,aktualy,aktualz, c=colors);


    ax.text(1,1,1,int(float(frek[2])))

    X = 1.2*pos[0,:]
    Y = 1.2*pos[1,:]
    Z = 1.2*pos[2,:]


# Create cubic bounding box to simulate equal aspect ratio
    max_range = np.array([X.max()-X.min(), Y.max()-Y.min(), Z.max()-Z.min()]).max()
    Xb = 0.5*max_range*np.mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(X.max()+X.min())
    Yb = 0.5*max_range*np.mgrid[-1:2:2,-1:2:2,-1:2:2][1].flatten() + 0.5*(Y.max()+Y.min())
    Zb = 0.5*max_range*np.mgrid[-1:2:2,-1:2:2,-1:2:2][2].flatten() + 0.5*(Z.max()+Z.min())
# Comment or uncomment following both lines to test the fake bounding box:
    for xb, yb, zb in zip(Xb, Yb, Zb):
       ax.plot([xb], [yb], [zb], 'w')




cmap = plt.cm.get_cmap('Greens')




N = 100
data = [ np.array(list(gen(N,ktery))) for ktery in range(numatoms)]
lines = [ax.plot([], [], [],c=colors[i])[0] for i in range(numatoms)]



aktual = [data[l][6][0] for l in range(numatoms) ]

print (aktual)

print('data:',np.shape(data),'lines:',np.shape(lines))








ani = animation.FuncAnimation(fig, update,40, fargs=(data, lines), interval=100/N, blit=False)
#ani.save('matplot004.jpg', writer='imagemagick',dpi=200)
plt.show()


#writervideo = animation.FFMpegWriter(fps=60)
#ani.save('out.avi', writer=writervideo)



fig.savefig('test.png',dpi=200)


f.close()
